import React, { FC } from "react";
import { TestProps } from "@cs-bosch-medusa/ui";
export type ChartTileContainerProps = TestProps & {
    /**
     * The activated tiles id list.
     * If the `ChartTile` id is not in the list then the tile is deactivated.
     */
    activatedTiles: string[];
    /**
     * Multiple `ChartTile` components passed as children.
     *
     * _Note:_ At least one children have to be passed to
     * the component in order to show it.
     */
    children: React.ReactNode;
    /**
     * Callback to be called when a tile is pressed.
     * The new activated tiles id list is passed as a parameter to the callback.
     */
    onChange(newActivatedTiles: string[]): void;
    /**
     * If `true`, tapping the indicator dots will trigger a scroll
     * to the page represented by the pressed dot.
     *
     * Defaults to `true`.
     */
    indicatorPressable?: boolean;
    /**
     * The maximum number of tiles to be shown in the container.
     * This can have 6 or 8 as value.
     *
     * Defaults to 8.
     */
    contentLimit?: 6 | 8;
};
export declare const ChartTileContainer: FC<ChartTileContainerProps>;
//# sourceMappingURL=chart-tile-container.d.ts.map