"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChartTile = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const ui_1 = require("@cs-bosch-medusa/ui");
const chart_tile_label_1 = require("./chart-tile-label");
const theme_1 = require("../../theme");
const tile_constants_1 = require("./tile-constants");
const LABEL_CONTAINER_PADDING = 2;
const VALUE_LABEL_FONT_SIZE = (0, ui_1.verticalScale)(16);
const styles = react_native_1.StyleSheet.create({
    container: {
        flex: 1,
    },
    labelContainer: {
        flex: 4,
        justifyContent: "flex-end",
        paddingBottom: LABEL_CONTAINER_PADDING,
        paddingHorizontal: ui_1.METRICS.CONTAINER.halfBasePadding,
    },
    middleContainer: {
        /**
         * NOTE: there was a request from the client that
         * the value labels should be aligned in one line
         * across the tiles.
         */
        flex: 5,
        justifyContent: "flex-end",
        paddingHorizontal: ui_1.METRICS.CONTAINER.halfBasePadding,
        paddingTop: LABEL_CONTAINER_PADDING,
    },
    middleRowContainer: {
        flexDirection: "row",
        justifyContent: "center",
    },
    unit: {
        paddingLeft: 2,
    },
    value: {
        paddingTop: (0, ui_1.verticalScale)(ui_1.METRICS.CONTAINER.halfBasePadding / 2),
    },
});
const ChartTile = (props) => {
    const { chartTheme } = (0, theme_1.useChartTheme)();
    const [maxNumberOfLabelLines, setMaxNumberOfLabelLines] = (0, react_1.useState)(chartTheme.tile.maxLabelRowHeightLimit);
    const { id, value, unit, label, color, activated = true, } = props;
    const tileColor = activated ? color : chartTheme.tile.deactivatedColor;
    const labelColor = activated
        ? chartTheme.tile.labelColor
        : chartTheme.tile.deactivatedColor;
    /**
     * Needed because on Android the shortening of texts based on the available space is not working. It always renders the number of lines passed through the `numberOfLines` prop.
     * For example, in case the value of the prop is 2 and there is space only for 1 line, the text is not shortened accordingly.
     * Inside this `onLayout`, the maximum number of lines to be shown is calculated based on the available space for the label and the number of lines that can fit inside it.
     */
    const onLayout = (0, react_1.useCallback)((event) => {
        if (react_native_1.Platform.OS === "android" &&
            event.nativeEvent.layout.height > 0) {
            // The height available for the text: the height of the container view - the padding on the bottom of it
            const heightForLabel = event.nativeEvent.layout.height - LABEL_CONTAINER_PADDING;
            setMaxNumberOfLabelLines(Math.min(chartTheme.tile.maxLabelRowHeightLimit, Math.trunc(heightForLabel / tile_constants_1.CHART_TILE_LABEL_FONT_SIZE)));
        }
    }, [chartTheme.tile.maxLabelRowHeightLimit]);
    return (react_1.default.createElement(react_native_1.View, { style: [
            styles.container,
            {
                backgroundColor: chartTheme.tile.backgroundColor,
                borderColor: tileColor,
                borderTopWidth: chartTheme.tile.topLineSize,
            },
        ] },
        react_1.default.createElement(react_native_1.View, { style: styles.middleContainer },
            react_1.default.createElement(react_native_1.View, { style: styles.middleRowContainer },
                react_1.default.createElement(ui_1.Label, { style: [
                        styles.value,
                        {
                            color: tileColor,
                            fontSize: VALUE_LABEL_FONT_SIZE,
                        },
                    ] }, value),
                react_1.default.createElement(chart_tile_label_1.ChartTileLabel, { style: [styles.unit, { color: tileColor }] }, unit))),
        react_1.default.createElement(react_native_1.View, { onLayout: onLayout, style: styles.labelContainer, testID: `${id}.labelView` },
            react_1.default.createElement(chart_tile_label_1.ChartTileLabel, { numberOfLines: maxNumberOfLabelLines, style: { color: labelColor } }, label))));
};
exports.ChartTile = ChartTile;
