"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RealTimeLineChart = void 0;
const react_1 = __importStar(require("react"));
const d3Scale = __importStar(require("d3-scale"));
const react_native_svg_1 = require("react-native-svg");
const lodash_1 = require("lodash");
const ui_1 = require("@cs-bosch-medusa/ui");
const theme_1 = require("../../theme");
const real_time_line_chart_metrics_1 = require("./real-time-line-chart-metrics");
const real_time_line_chart_x_axis_labels_1 = require("./real-time-line-chart-x-axis-labels");
const real_time_line_chart_y_axis_labels_1 = require("./real-time-line-chart-y-axis-labels");
const real_time_line_chart_grid_1 = require("./real-time-line-chart-grid");
const real_time_line_chart_clip_path_1 = require("./real-time-line-chart-clip-path");
const real_time_line_chart_lines_1 = require("./real-time-line-chart-lines");
const real_time_line_chart_points_1 = require("./real-time-line-chart-points");
const utils_1 = require("../../utils");
const LINE_CHART_TEXT_SIZE = 4;
const DEFAULT_AXIS_TICKS_NUMBER = 5;
/**
 * Real-time based line chart svg component.
 */
exports.RealTimeLineChart = react_1.default.forwardRef(({ width, height, values, activeParameters, formatDate, formatValue, colors, xTicksNumber, yTicksNumber, showPoints = true, defaultIntervalRange, initialIntervalRange, interval = 0, rightSideTimeShift = 0, withBackground = false, testID, }, ref) => {
    // eslint-disable-next-line unicorn/no-null
    const svgRef = (0, react_1.useRef)(null);
    const toBase64 = (0, react_1.useCallback)(() => new Promise((resolve) => {
        /**
         * NOTE: Type definitions are missing for the Svg component's ref `toDataURL` method.
         *
         * Reference: https://github.com/react-native-community/react-native-svg/issues/1408
         */
        svgRef.current.toDataURL((base64) => {
            resolve(base64);
        });
    }), [svgRef]);
    (0, react_1.useImperativeHandle)(ref, () => ({
        toBase64,
    }), [toBase64]);
    const { chartTheme } = (0, theme_1.useChartTheme)();
    const { xMin, xMax, yMin, yMax } = (0, react_1.useMemo)(() => getRange(values, initialIntervalRange, defaultIntervalRange, interval), [values, initialIntervalRange, defaultIntervalRange, interval]);
    const xMaximum = xMax + rightSideTimeShift;
    const xScale = (0, react_1.useMemo)(() => d3Scale
        .scaleTime()
        .domain([xMin, xMaximum])
        .range([
        real_time_line_chart_metrics_1.REAL_TIME_LINE_CHART_METRICS.LEFT_PADDING,
        width - real_time_line_chart_metrics_1.REAL_TIME_LINE_CHART_METRICS.RIGHT_PADDING,
    ]), [xMin, xMaximum, width]);
    const yScale = (0, react_1.useMemo)(() => d3Scale
        .scaleLinear()
        .domain([yMin, yMax + LINE_CHART_TEXT_SIZE])
        .range([
        height - real_time_line_chart_metrics_1.REAL_TIME_LINE_CHART_METRICS.BOTTOM_PADDING,
        real_time_line_chart_metrics_1.REAL_TIME_LINE_CHART_METRICS.TOP_PADDING,
    ]), [yMin, yMax, height]);
    const axisTickTimestamps = (0, react_1.useMemo)(() => getAxisValues(xMin, xMaximum, xTicksNumber), [xMin, xMaximum, xTicksNumber]);
    const yAxisTickValues = (0, react_1.useMemo)(() => getAxisValues(yMin, yMax, yTicksNumber), [yMin, yMax, yTicksNumber]);
    return (react_1.default.createElement(react_native_svg_1.Svg, { height: height, ref: svgRef, style: {
            backgroundColor: chartTheme.realTimeLineChart.backgroundColor,
        }, ...(0, ui_1.getTestIdProps)(testID), width: width },
        withBackground && (react_1.default.createElement(react_native_svg_1.Rect, { fill: chartTheme.realTimeLineChart.backgroundColor, height: height, width: width })),
        react_1.default.createElement(real_time_line_chart_x_axis_labels_1.RealTimeLineChartXAxisLabels, { axisTickTimestamps: axisTickTimestamps, formatDate: formatDate, height: height, scale: xScale }),
        react_1.default.createElement(real_time_line_chart_y_axis_labels_1.RealTimeLineChartYAxisLabels, { axisTickValues: yAxisTickValues, formatValue: formatValue, scale: yScale }),
        react_1.default.createElement(real_time_line_chart_grid_1.RealTimeLineChartGrid, { height: height, width: width, xAxisTickTimestamps: axisTickTimestamps, xScale: xScale, yAxisTickValues: yAxisTickValues, yScale: yScale }),
        react_1.default.createElement(real_time_line_chart_clip_path_1.RealTimeLineChartClipPath, { height: height, width: width, xMin: xMin, xScale: xScale, yMax: yMax, yScale: yScale },
            react_1.default.createElement(real_time_line_chart_lines_1.RealTimeLineChartLines, { activeParameters: activeParameters, colors: colors, values: values, xScale: xScale, yScale: yScale }),
            showPoints && (react_1.default.createElement(real_time_line_chart_points_1.RealTimeLineChartPoints, { activeParameters: activeParameters, colors: colors, values: values, xScale: xScale, yScale: yScale })))));
});
/**
 * Validates and generates axis tick values.
 * If the ticksNumber is less or equal to 0 then
 * the DEFAULT_AXIS_TICKS_NUMBER is used and a warning message is shown.
 *
 * @param min interval range minimum value
 * @param max interval range maximum value
 * @param ticks ticks number to generate
 */
const getAxisValues = (min, max, ticks) => {
    let ticksNumber = ticks;
    if (ticksNumber < 2) {
        console.warn("At least 2 ticksNumber is required to be given. Fallback to default ticks number:", DEFAULT_AXIS_TICKS_NUMBER);
        ticksNumber = DEFAULT_AXIS_TICKS_NUMBER;
    }
    return (0, lodash_1.range)(min, max + 1, (max - min) / (ticksNumber - 1));
};
/**
 * @description Helper function in order to determine
 * the axis interval range of the line chart.
 *
 * @param lineChartValues line chart values
 * @param initialIntervalRange initial interval range of the line chart
 * @param defaultIntervalRange default interval range of the line chart
 *
 * @returns `RealTimeLineChartIntervalRange` object
 */
const getRange = (lineChartValues, initialIntervalRange, defaultIntervalRange, interval) => {
    var _a, _b, _c, _d;
    const calculateRange = Object.values(lineChartValues).some((values) => values.length > 1);
    if (calculateRange) {
        // eslint-disable-next-line unicorn/prefer-spread
        const concatenatedValues = [].concat(...Object.values(lineChartValues));
        let xMin = concatenatedValues[0].timestamp;
        const firstNumericElement = concatenatedValues.find((element) => (0, utils_1.isNumber)(element.value));
        let yMin = firstNumericElement !== undefined
            ? firstNumericElement.value
            : initialIntervalRange.yMin;
        let xMax = xMin;
        let yMax = yMin;
        for (let i = 1; i < concatenatedValues.length; i++) {
            const temp = concatenatedValues[i];
            if (xMin > temp.timestamp) {
                xMin = temp.timestamp;
            }
            if (xMax < temp.timestamp) {
                xMax = temp.timestamp;
            }
            if ((0, utils_1.isNumber)(temp.value)) {
                if (yMin > temp.value) {
                    yMin = temp.value;
                }
                if (yMax < temp.value) {
                    yMax = temp.value;
                }
            }
        }
        if (yMax === yMin && yMin === initialIntervalRange.yMin) {
            yMax = initialIntervalRange.yMax;
        }
        yMin = Math.floor(yMin);
        yMax = Math.ceil(yMax);
        xMin = interval > 0 ? xMax - interval : xMin;
        if (defaultIntervalRange) {
            return {
                xMin: (_a = defaultIntervalRange.xMin) !== null && _a !== void 0 ? _a : xMin,
                xMax: (_b = defaultIntervalRange.xMax) !== null && _b !== void 0 ? _b : xMax,
                yMin: (_c = defaultIntervalRange.yMin) !== null && _c !== void 0 ? _c : yMin,
                yMax: (_d = defaultIntervalRange.yMax) !== null && _d !== void 0 ? _d : yMax,
            };
        }
        return {
            xMin,
            xMax,
            yMin,
            yMax,
        };
    }
    return initialIntervalRange;
};
