import React from "react";
import { TestProps } from "@cs-bosch-medusa/ui";
import { CalendarRef } from "./calendar-ref";
import { ListResolution } from "./types";
export type CalendarListPickerProps = TestProps & {
    /**
     * The earliest date to which to go back to when generating the list.
     */
    earliestDate: Date;
    /**
     * The date of the first day of the selected month/year depending on the
     * resolution.
     */
    periodStart: Date;
    /** Latest date to include in the list. Defaults to now. */
    maxDate?: Date;
    /** Whether to generate a list of months or years. */
    resolution: ListResolution;
    /**
     * Formatter to create the labels for the items based on the date and resolution.
     * It also receives the selected state in case the function returns custom components.
     *
     * The result is always wrapped in a label, so if you decide to render custom Text components,
     * make sure to only overwrite styles you need to. E.g leaving the color unspecified will
     * allow the component to still control the text color.
     *
     * For performance reasons, make sure this function is memoized.
     */
    formatLabel(date: Date, resolution: ListResolution, selected: boolean): string | JSX.Element;
    /**
     * Callback to be invoked when selected period changes.
     * The date of the first day of the period will be passed.
     */
    onChange(startDate: Date): void;
};
export declare const CalendarListPicker: React.ForwardRefExoticComponent<TestProps & {
    /**
     * The earliest date to which to go back to when generating the list.
     */
    earliestDate: Date;
    /**
     * The date of the first day of the selected month/year depending on the
     * resolution.
     */
    periodStart: Date;
    /** Latest date to include in the list. Defaults to now. */
    maxDate?: Date | undefined;
    /** Whether to generate a list of months or years. */
    resolution: ListResolution;
    /**
     * Formatter to create the labels for the items based on the date and resolution.
     * It also receives the selected state in case the function returns custom components.
     *
     * The result is always wrapped in a label, so if you decide to render custom Text components,
     * make sure to only overwrite styles you need to. E.g leaving the color unspecified will
     * allow the component to still control the text color.
     *
     * For performance reasons, make sure this function is memoized.
     */
    formatLabel(date: Date, resolution: ListResolution, selected: boolean): string | JSX.Element;
    /**
     * Callback to be invoked when selected period changes.
     * The date of the first day of the period will be passed.
     */
    onChange(startDate: Date): void;
} & React.RefAttributes<CalendarRef>>;
//# sourceMappingURL=calendar-list-picker.d.ts.map