"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCalendarPickerData = void 0;
const react_1 = require("react");
const moment_1 = __importDefault(require("moment"));
const calendar_settings_1 = require("./calendar-settings");
const useCalendarPickerData = ({ selectedDay, minDate, maxDate, disableNavigationToDisabledMonths = false, }) => {
    const [currentMonth, setCurrentMonth] = (0, react_1.useState)(() => (0, moment_1.default)(selectedDay).format(calendar_settings_1.CALENDAR_TIME_FORMAT));
    const selectedDayString = (0, react_1.useMemo)(() => (0, moment_1.default)(selectedDay).format(calendar_settings_1.CALENDAR_TIME_FORMAT), [selectedDay]);
    /**
     * Called whenever the currently visible month changes. With this configuration
     * only one month can be displayed fully at once.
     */
    const onVisibleMonthsChange = (0, react_1.useCallback)((months) => {
        setCurrentMonth(months[0].dateString);
    }, []);
    const maxDateString = (0, react_1.useMemo)(() => maxDate ? (0, moment_1.default)(maxDate).format(calendar_settings_1.CALENDAR_TIME_FORMAT) : undefined, [maxDate]);
    const minDateString = (0, react_1.useMemo)(() => minDate ? (0, moment_1.default)(minDate).format(calendar_settings_1.CALENDAR_TIME_FORMAT) : undefined, [minDate]);
    const disabledArrowLeft = disableNavigationToDisabledMonths &&
        minDateString !== undefined &&
        // checks if the currently shown month is same or after the minimum date's month
        // in this format the years are also checked:
        // 2022.12.30 < 2023.01.01  - is true
        (0, moment_1.default)(currentMonth).isSameOrBefore(minDateString, "month");
    const disabledArrowRight = disableNavigationToDisabledMonths &&
        maxDateString !== undefined &&
        (0, moment_1.default)(currentMonth).isSameOrAfter(maxDateString, "month");
    return {
        disabledArrowLeft,
        disabledArrowRight,
        maxDateString,
        minDateString,
        selectedDayString,
        onVisibleMonthsChange,
    };
};
exports.useCalendarPickerData = useCalendarPickerData;
