"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CalendarPickerWeek = void 0;
const react_1 = __importStar(require("react"));
const react_native_calendars_1 = require("react-native-calendars");
const moment_1 = __importDefault(require("moment"));
const ui_1 = require("@cs-bosch-medusa/ui");
const calendar_settings_1 = require("./calendar-settings");
const calendar_arrow_1 = require("./calendar-arrow");
const calendar_ref_1 = require("./calendar-ref");
const theme_1 = require("../../theme");
const header_text_1 = require("./header-text");
const calendar_picker_data_hook_1 = require("./calendar-picker-data-hook");
exports.CalendarPickerWeek = (0, react_1.forwardRef)(({ weekStart, onWeekChange, continuousSelection = false, markToday = true, ...props }, ref) => {
    const { chartTheme } = (0, theme_1.useChartTheme)();
    const { theme } = (0, ui_1.useTheme)();
    const { selectedDayString: weekStartString, minDateString, maxDateString, onVisibleMonthsChange, disabledArrowLeft, disabledArrowRight, } = (0, calendar_picker_data_hook_1.useCalendarPickerData)({
        ...props,
        selectedDay: weekStart,
    });
    const markedDates = (0, react_1.useMemo)(() => {
        const weekStartMoment = (0, moment_1.default)(weekStartString);
        const markedDates = {};
        for (let i = 0; i < 7; ++i) {
            const day = weekStartMoment.clone().add(i, "days");
            markedDates[day.format(calendar_settings_1.CALENDAR_TIME_FORMAT)] = {
                startingDay: continuousSelection ? i == 0 : true,
                endingDay: continuousSelection ? i == 6 : true,
                color: chartTheme.resolution.selectedColor,
                textColor: chartTheme.resolution.selectedTextColor,
                customContainerStyle: {
                    borderWidth: 0,
                },
            };
        }
        return markedDates;
    }, [
        chartTheme.resolution.selectedColor,
        chartTheme.resolution.selectedTextColor,
        continuousSelection,
        weekStartString,
    ]);
    const onDayPress = (0, react_1.useCallback)((selectedDateObj) => {
        const selectedDate = (0, moment_1.default)(selectedDateObj.timestamp);
        onWeekChange(selectedDate.startOf("week").toDate());
    }, [onWeekChange]);
    const { initialMonth } = (0, calendar_ref_1.useCalendarRef)(ref, weekStartString, onDayPress);
    const calendarTheme = (0, react_1.useMemo)(() => ({
        ...(0, calendar_settings_1.getCalendarTheme)(chartTheme, theme),
        // Fix for period borders on android
        // see: https://github.com/wix/react-native-calendars/pull/728#issuecomment-504947907
        // Issue: https://github.com/wix/react-native-calendars/issues/1321
        "stylesheet.day.period": {
            base: {
                ...calendar_settings_1.baseCellStyle,
                overflow: "hidden",
            },
            ...(markToday && { today: (0, calendar_settings_1.getTodayStyle)(chartTheme) }),
            ...(0, calendar_settings_1.getPeriodStyles)(chartTheme, theme),
        },
    }), [chartTheme, markToday, theme]);
    const renderArrow = (0, react_1.useCallback)((direction) => (react_1.default.createElement(calendar_arrow_1.CalendarArrow, { direction: direction, disabled: (direction == "left" && disabledArrowLeft) ||
            (direction == "right" && disabledArrowRight) })), [disabledArrowLeft, disabledArrowRight]);
    return (react_1.default.createElement(react_native_calendars_1.Calendar, { disableArrowLeft: disabledArrowLeft, disableArrowRight: disabledArrowRight, 
        // eslint-disable-next-line import/no-named-as-default-member
        firstDay: moment_1.default.localeData().firstDayOfWeek(), 
        // Decides which month to display on mount. Also refreshes display
        // on change.
        initialDate: initialMonth, markedDates: markedDates, markingType: "period", maxDate: maxDateString, minDate: minDateString, onDayPress: onDayPress, onVisibleMonthsChange: onVisibleMonthsChange, renderArrow: renderArrow, renderHeader: renderHeader, style: calendar_settings_1.calendarStyle, theme: calendarTheme }));
});
const renderHeader = (month) => (react_1.default.createElement(header_text_1.HeaderText, { month: month }));
