"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyboardContainer = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const styles = react_native_1.StyleSheet.create({
    container: {
        flex: 1,
    },
});
/**
 * A container component which resolves the input field covered by the Keyboard issue on iOS.
 *
 * If the keyboard covers an input field on Android then check if the `windowSoftInputMode`
 * is properly set in the `AndroidManifest.xml` file. Android documentation can be found
 * [here](https://developer.android.com/guide/topics/manifest/activity-element#wsoft).
 *
 * If the keyboard covers the selected input field on iOS then check if the `headerHeight`
 * property is correctly set to the distance between the top of the screen and the
 * container and make sure that the children are nested into scrollable component.
 */
const KeyboardContainer = ({ style, headerHeight, children, }) => {
    const containerStyle = react_native_1.StyleSheet.flatten([styles.container, style]);
    return react_native_1.Platform.select({
        ios: (react_1.default.createElement(react_native_1.KeyboardAvoidingView, { behavior: "padding", keyboardVerticalOffset: headerHeight, style: containerStyle }, children)),
        /**
         * NOTE: on Android the children property it's wrapped in a view,
         * so the same style applies for the container as it is on iOS.
         */
        android: react_1.default.createElement(react_native_1.View, { style: containerStyle }, children),
        // eslint-disable-next-line unicorn/no-null
        default: null,
    });
};
exports.KeyboardContainer = KeyboardContainer;
