"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListItem = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const icons_1 = require("../icons");
const separators_1 = require("../separators");
const text_1 = require("../text");
const list_item_props_1 = require("./list-item-props");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    labelWithoutIcon: {
        paddingLeft: theme_1.METRICS.ICON.basePadding,
    },
    largePadding: {
        paddingVertical: theme_1.METRICS.CONTAINER.doubleBasePadding,
    },
    leftItem: {
        paddingRight: theme_1.METRICS.CONTAINER.basePadding,
    },
    mainLabelWrapper: {
        flex: 1,
    },
    rightItem: {
        paddingLeft: theme_1.METRICS.CONTAINER.basePadding,
    },
    status: {
        maxWidth: "50%",
        paddingLeft: theme_1.METRICS.CONTAINER.basePadding,
    },
    statusRight: {
        paddingRight: theme_1.METRICS.ICON.basePadding,
    },
    textContentWrapper: {
        alignItems: "center",
        flexDirection: "row",
        flex: 1,
    },
    wrapper: {
        alignItems: "center",
        flexDirection: "row",
        padding: theme_1.METRICS.CONTAINER.basePadding,
    },
});
/**
 * The items that make up a list. All content-related properties are optional,
 * so it's up to you to make sure that you don't create an empty list item.
 *
 * Through the `leftComponent` and `rightComponent` properties you can pass
 * any JSX element to customize the look and functionality of the item,
 * such as icons, switches, checkboxes, or any other elements for custom
 * behavior.
 *
 * If an `onPress` callback is provided, the list item becomes touchable.
 */
const ListItem = ({ text, description, status, disabled = false, leftComponent, middleComponent, rightComponent, arrow, separator = true, onPress, textColor, spacing = list_item_props_1.ListItemSpacing.NORMAL, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    let primaryTextColor = theme.textColor;
    let secondaryTextColor = theme.listSecondaryColor;
    if (disabled) {
        primaryTextColor = theme.disabledColor;
        secondaryTextColor = theme.disabledColor;
    }
    else if (textColor) {
        primaryTextColor = textColor;
        secondaryTextColor = textColor;
    }
    const paddingVertical = spacing === list_item_props_1.ListItemSpacing.LARGE ? styles.largePadding : undefined;
    return (react_1.default.createElement(react_native_1.Pressable, { disabled: disabled || !onPress, onPress: onPress, ...(0, util_1.getTestIdProps)(`${testID}.touch`) }, ({ pressed }) => {
        const { fillColor, lineColor } = getListItemColors(theme, {
            pressed,
            disabled,
        });
        return (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(react_native_1.View, { style: [
                    styles.wrapper,
                    {
                        backgroundColor: fillColor,
                        ...paddingVertical,
                    },
                ], ...(0, util_1.getTestIdProps)(testID) },
                leftComponent && (react_1.default.createElement(react_native_1.View, { style: styles.leftItem }, leftComponent)),
                react_1.default.createElement(react_native_1.View, { style: styles.textContentWrapper },
                    react_1.default.createElement(react_native_1.View, { style: [
                            styles.mainLabelWrapper,
                            !leftComponent &&
                                styles.labelWithoutIcon,
                        ] }, middleComponent !== null && middleComponent !== void 0 ? middleComponent : (react_1.default.createElement(react_1.default.Fragment, null,
                        text !== undefined && (react_1.default.createElement(text_1.Label, { style: {
                                color: primaryTextColor,
                            } }, text)),
                        description !== undefined && (react_1.default.createElement(text_1.Subtitle, { style: {
                                color: secondaryTextColor,
                            } }, description))))),
                    status !== undefined && (react_1.default.createElement(text_1.Subtitle, { style: [
                            styles.status,
                            !rightComponent && !arrow
                                ? styles.statusRight
                                : undefined,
                            { color: primaryTextColor },
                        ] }, status))),
                rightComponent && (react_1.default.createElement(react_native_1.View, { style: styles.rightItem }, rightComponent)),
                arrow && (react_1.default.createElement(react_native_1.View, { style: styles.rightItem },
                    react_1.default.createElement(icons_1.ForwardRightIcon, { fillColor: primaryTextColor, iconSize: icons_1.IconSize.XSMALL })))),
            separator && (react_1.default.createElement(separators_1.Separator, { style: { backgroundColor: lineColor } }))));
    }));
};
exports.ListItem = ListItem;
const getListItemColors = (theme, { pressed, disabled }) => {
    if (disabled) {
        return theme.list.disabled;
    }
    return pressed ? theme.list.pressed : theme.list.default;
};
