import React from "react";
import { NotificationProps } from "./notification";
export type NotificationConfig = {
    /**
     * Specifies the duration (in seconds) for which the notification is visible.
     * Must be a positive value. Defaults to 5 seconds.
     */
    readonly duration?: number;
    /**
     * Whether the notification should disappear after the time specified in `duration`.
     * Defaults to `true`.
     */
    readonly autoHide?: boolean;
};
/**
 * Return type of the useNotification hook.
 */
export type NotificationResult = {
    /**
     * Notification component to place in the render function.
     * Message and notification type should be passed to this component.
     */
    Notification: React.ComponentType<NotificationProps>;
    /**
     * Make the notification visible. If not specified otherwise in the config,
     * the notification will fade out after 5 seconds.
     *
     * @param config configuration object to modify behavior.
     */
    show(config?: NotificationConfig): void;
    /** Make the notification fade out and disappear. */
    hide(): void;
};
/**
 * Returns an object with a constant reference that contains the Notification
 * component as well as the show and hide methods.
 */
export declare const useNotification: () => Readonly<NotificationResult>;
//# sourceMappingURL=notification-hook.d.ts.map