"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Notification = exports.NotificationVariant = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const containers_1 = require("../containers");
const text_1 = require("../text");
const icons_1 = require("../icons");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    notification: {
        flexDirection: "row",
    },
    notificationText: {
        alignSelf: "center",
        flex: 1,
        paddingHorizontal: theme_1.METRICS.CONTAINER.basePadding,
    },
});
var NotificationVariant;
(function (NotificationVariant) {
    NotificationVariant["INFO"] = "INFO";
    NotificationVariant["SUCCESS"] = "SUCCESS";
    NotificationVariant["WARNING"] = "WARNING";
    NotificationVariant["ERROR"] = "ERROR";
})(NotificationVariant = exports.NotificationVariant || (exports.NotificationVariant = {}));
const Notification = ({ message, variant, icon: CustomIcon, onDismiss, showDismissButton = true, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const [VariantIcon, signalStyle] = getNotificationVariantStyle(variant, theme);
    const mainContentStyle = react_native_1.StyleSheet.flatten([
        styles.notification,
        {
            backgroundColor: signalStyle.fillColor,
        },
    ]);
    const textStyle = react_native_1.StyleSheet.flatten([
        styles.notificationText,
        {
            color: signalStyle.textColor,
        },
    ]);
    const Icon = CustomIcon !== null && CustomIcon !== void 0 ? CustomIcon : VariantIcon;
    return (react_1.default.createElement(containers_1.Container, { style: mainContentStyle, testID: testID },
        react_1.default.createElement(Icon, { fillColor: signalStyle.textColor, iconSize: icons_1.IconSize.XSMALL }),
        react_1.default.createElement(text_1.Subtitle, { style: textStyle }, message),
        showDismissButton && (react_1.default.createElement(react_native_1.TouchableOpacity, { activeOpacity: theme.activeOpacity, onPress: onDismiss, ...(0, util_1.getTestIdProps)(`${testID}.dismiss`) },
            react_1.default.createElement(icons_1.CloseIcon, { fillColor: signalStyle.textColor, iconSize: icons_1.IconSize.XSMALL })))));
};
exports.Notification = Notification;
const getNotificationVariantStyle = (variant, theme) => {
    switch (variant) {
        case NotificationVariant.INFO:
            return [icons_1.AlertInfoIcon, theme.info];
        case NotificationVariant.SUCCESS:
            return [icons_1.AlertSuccessIcon, theme.success];
        case NotificationVariant.WARNING:
            return [icons_1.AlertWarningIcon, theme.warning];
        case NotificationVariant.ERROR:
            return [icons_1.AlertErrorIcon, theme.error];
    }
};
