"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessIndicatorDots = void 0;
const lodash_1 = require("lodash");
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const react_native_svg_1 = require("react-native-svg");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    indicatorWrapper: {
        flexDirection: "row",
        justifyContent: "center",
        margin: theme_1.METRICS.CONTAINER.baseMargin,
    },
});
const radius = 3;
const activeRadius = 5;
const spacing = 22;
const circleSize = activeRadius * 2;
const windowWidth = react_native_1.Dimensions.get("window").width;
/**
 * Component used for displaying the status of a process.
 * Stages are represented as dots and the current stage
 * is highlighted.
 */
const ProcessIndicatorDots = ({ maxStages, maxWidth = windowWidth - theme_1.METRICS.SCREEN.basePadding * 2, currentStage, style, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const stages = (0, lodash_1.range)(1, maxStages + 1);
    const width = spacing * (maxStages - 1) + 2 * activeRadius;
    const svgWidth = Math.min(width, maxWidth);
    const resultingStyle = react_native_1.StyleSheet.flatten([styles.indicatorWrapper, style]);
    return (
    // eslint-disable-next-line react/jsx-props-no-spreading
    react_1.default.createElement(react_native_1.View, { style: resultingStyle, ...(0, util_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_svg_1.Svg, { height: circleSize, viewBox: `0 0 ${width} ${circleSize}`, width: svgWidth }, stages.map((stage, index) => (react_1.default.createElement(react_native_svg_1.Circle, { fill: stage === currentStage
                ? theme.primaryColor
                : theme.disabledColor, key: stage, r: stage === currentStage ? activeRadius : radius, x: index * spacing + activeRadius, y: activeRadius }))))));
};
exports.ProcessIndicatorDots = ProcessIndicatorDots;
