"use strict";
// src: https://www.benjaminjohnson.me/blog/testing-animations-in-react-native/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.timeTravelRealTime = exports.timeTravel = exports.cleanupTimeTravel = exports.setupTimeTravel = void 0;
// eslint-disable-next-line import/no-extraneous-dependencies
const mockdate_1 = __importDefault(require("mockdate"));
const FRAME_TIME = 10;
/**
 * Call this in `beforeEach` to initialize logic needed
 * for testing animations.
 *
 * _Note:_ this modifies the current date and uses fake timers.
 */
function setupTimeTravel() {
    mockdate_1.default.set(0);
    jest.useFakeTimers({ legacyFakeTimers: true });
    //@ts-ignore
    global.requestAnimationFrame = (cb) => {
        setTimeout(cb, FRAME_TIME);
    };
}
exports.setupTimeTravel = setupTimeTravel;
/**
 * Call this in `afterEach` to clean up.
 */
function cleanupTimeTravel() {
    jest.clearAllTimers();
}
exports.cleanupTimeTravel = cleanupTimeTravel;
function advanceOneFrame() {
    const now = Date.now();
    mockdate_1.default.set(new Date(now + FRAME_TIME));
    jest.advanceTimersByTime(FRAME_TIME);
}
/**
 * Advance time by specified ms in a way that Animated will
 * recognize. Use this to test animations. By passing their duration
 * as parameter you can skip to the end of the animation.
 *
 * _Note:_ The actual time in timers will not be advanced by
 * the milliseconds specified. If you want to test other timers
 * besides animation, use `timeTravelRealTime`.
 */
function timeTravel(msToAdvance = FRAME_TIME) {
    const numberOfFramesToRun = msToAdvance / FRAME_TIME;
    let framesElapsed = 0;
    // Step through each of the frames until we've ran them all
    while (framesElapsed < numberOfFramesToRun) {
        advanceOneFrame();
        framesElapsed++;
    }
}
exports.timeTravel = timeTravel;
/**
 * Alternative to `timeTravel`. Use this if you want to advance
 * other timers besides animation by the time specified as parameter.
 */
function timeTravelRealTime(msToAdvance) {
    timeTravel(msToAdvance * FRAME_TIME);
}
exports.timeTravelRealTime = timeTravelRealTime;
