"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.darkenColor = void 0;
const colors_1 = require("./colors");
/**
 * Changes the brightness of the provided color.
 * @param originalColor the original color which must be in hexadecimal format. Accepted formats: #ea2 or #23ae9e.
 * @param scale a number which defines how to calculate the shade of the original color
 * must be between [-1, 1]. If the scale is [0, 1] -> it will calculate the `shade` - a darker color.
 * if it's lower than 0 then returns a `tint color` - lighter color.
 *
 * @example darkenColor("#EA0016", 0.2)
 *  // returns a 20% darker red
 * @example darkenColor("#EA0016", -0.6)
 *  // returns a 60% lighter red
 */
const darkenColor = (originalColor, scale) => {
    if (scale <= -1) {
        return colors_1.Colors.WHITE;
    }
    if (scale >= 1) {
        return colors_1.Colors.BLACK;
    }
    if (scale == 0) {
        return originalColor;
    }
    let currentR;
    let currentG;
    let currentB;
    if (originalColor.length == 7) {
        currentR = scaleValue(originalColor.slice(1, 3), scale);
        currentG = scaleValue(originalColor.slice(3, 5), scale);
        currentB = scaleValue(originalColor.slice(5), scale);
        return `#${currentR}${currentG}${currentB}`;
    }
    else if (originalColor.length === 4) {
        // in case of #fff or #e4a
        currentR = scaleValue(`${originalColor[1]}${originalColor[1]}`, scale);
        currentG = scaleValue(`${originalColor[2]}${originalColor[2]}`, scale);
        currentB = scaleValue(`${originalColor[3]}${originalColor[3]}`, scale);
        return `#${currentR}${currentG}${currentB}`;
    }
    return originalColor;
};
exports.darkenColor = darkenColor;
/**
 * NOTE: based on this answer https://stackoverflow.com/a/31325812/5644236
 * @param originalHexValue one of the  R, G, B colour hex. {[0-9A-F][0-9A-F]}
 * @param scale must be between [-1,1]
 * @returns a new hex as part of a brighter or darker color.
 */
const scaleValue = (originalHexValue, scale) => {
    const currentInt = Number.parseInt(originalHexValue, 16);
    let newInt;
    if (scale > 0) {
        newInt = Math.round(currentInt * (1 - scale));
    }
    else {
        const sc = -scale;
        newInt = Math.round(currentInt + (255 - currentInt) * sc);
    }
    const newHexValue = newInt < 16 ? `0${newInt.toString(16)}` : newInt.toString(16);
    return newHexValue;
};
