package com.bosch.tt.de.emma.connector.service;

import android.util.Log;

import com.bosch.sh.connector.thirdparty.api.http.HttpResponseHandler;
import com.bosch.sh.connector.thirdparty.api.http.Result;

import java.io.IOException;

import okhttp3.Response;
import okhttp3.ResponseBody;

public class EmmaHttpResponseHandler implements HttpResponseHandler {
	private static final String TAG = EmmaHttpResponseHandler.class.getSimpleName();

	private final ShcCallback shcCallback;
	private final String url;

	public EmmaHttpResponseHandler(String url, ShcCallback shcCallback) {
		this.url = url;
		this.shcCallback = shcCallback;
	}

	@Override
	public void onResponse(Response response) {
		String bodyString = "";
		try {
			ResponseBody responseBody = response.body();
			if (responseBody != null) {
				bodyString = responseBody.string();
			}
			
			Result result = new Result(response.code(), bodyString);
			shcCallback.onResult(result);
			
		} catch (IOException e) {
			Log.e(TAG, "Request has failed", e);
			shcCallback.onResult(new Result(0, "Request has failed. Url:" + this.url, e));
		}
	}

	@Override
	public void onFailure(Result result) {
		Log.e(TAG, "Error during the request. Result: { " + result.getMessage() + " " + result.getCode() + " }");
		shcCallback.onResult(result);
	}
}
