//
//  RNTTPPrivacyComponentManager.m
//  RNPrivacyComponent
//
//  Created by István Bege on 01/11/2018.
//  Copyright (C) 2018, Bosch Thermotechnik GmbH. All rights reserved.
//
#import <React/RCTViewManager.h>
#import <React/RCTLog.h>
#import <React/RCTUIManager.h>

#import "RNTTPrivacyComponent.h"
#import "ProviderManager.h"
#import "Assembler.h"

@interface RNTTPrivacyComponentManager : RCTViewManager
@end

@implementation RNTTPrivacyComponentManager

RCT_EXPORT_MODULE()

RCT_EXPORT_VIEW_PROPERTY(onDocumentLoaded, RCTBubblingEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onScrollEnded, RCTBubblingEventBlock)
RCT_CUSTOM_VIEW_PROPERTY(css, NSString, RNTTPrivacyComponent)
{
    [view setCustomCSS:[RCTConvert NSString:json]];
}

RCT_CUSTOM_VIEW_PROPERTY(activityIndicatorStyle, NSString, RNTTPrivacyComponent)
{
    NSString* prop = [RCTConvert NSString:json];
    UIActivityIndicatorViewStyle style;
    if ([prop isEqualToString:@"UIActivityIndicatorViewStyleWhiteLarge"]) {
        style = UIActivityIndicatorViewStyleWhiteLarge;
    } else {
        if ([prop isEqualToString:@"UIActivityIndicatorViewStyleWhite"]) {
            style = UIActivityIndicatorViewStyleWhite;
        } else {
            style = UIActivityIndicatorViewStyleGray;
        }
    }
    [view setActivityIndicatorViewStyle:style];
}

- (UIView *)view
{
    return [[RNTTPrivacyComponent alloc] init];
}

RCT_EXPORT_METHOD(loadPrivacyPolicy:(NSDictionary*)selectedCountry reactTag:(nonnull NSNumber *)reactTag)
{
    [self loadDocumentWithProvider:[[ProviderManager getInstance] privacyProvider] selectedCountry:selectedCountry reactTag:reactTag];
}

RCT_EXPORT_METHOD(loadTermsOfUse:(NSDictionary*)selectedCountry reactTag:(nonnull NSNumber *)reactTag)
{
    [self loadDocumentWithProvider:[[ProviderManager getInstance] termsOfUseProvider] selectedCountry:selectedCountry reactTag:reactTag];
}

- (void)loadDocumentWithProvider:(TTPPrivacyProvider*)provider selectedCountry:(NSDictionary*)selectedCountry reactTag:(nonnull NSNumber*)reactTag {
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *,RNTTPrivacyComponent *> *viewRegistry) {
        RNTTPrivacyComponent* view = viewRegistry[reactTag];
        if (![view isKindOfClass:[RNTTPrivacyComponent class]]) {
            RCTLogError(@"Invalid view returned from registry, expecting RNTTPPrivacyComponent, got: %@", view);
        } else {
            TTPCountry* country = [Assembler createCountry:selectedCountry];
            [view loadDocumentWithProvider:provider country:[country code] language:[country language] delegate:view];
        }
    }];
}

- (TTPPrivacyProvider*)getNeededPrivacyProvider:(BOOL)showTermsOfUse
{
    return showTermsOfUse == YES ? [[ProviderManager getInstance] termsOfUseProvider] : [[ProviderManager getInstance] privacyProvider];
}

@end

